#ifndef CC_RAE32STD_H_INCLUDED
#define CC_RAE32STD_H_INCLUDED 1

#include <e32std.h>
#include "raii.h"

class RAThread : public RThread, public RABase {
public:
	inline void CreateLA(const TDesC& aName,TThreadFunction aFunction,TInt aStackSize,TInt aHeapMinSize,TInt aHeapMaxSize,TAny *aPtr,TOwnerType aType=EOwnerProcess) {
		RABase::CloseRA();
		User::LeaveIfError(Create(aName,aFunction,aStackSize,aHeapMinSize,aHeapMaxSize,aPtr,aType));
		iOpen=ETrue;
		PutOnStackL();
	}
	inline void CreateLA(const TDesC& aName,TThreadFunction aFunction,TInt aStackSize,RHeap* aHeap,TAny* aPtr,TOwnerType aType=EOwnerProcess) {
		RABase::CloseRA();
		User::LeaveIfError(Create(aName,aFunction,aStackSize,aHeap,aPtr,aType));
		iOpen=ETrue;
		PutOnStackL();
	}
	inline void CreateLA(const TDesC& aName,TThreadFunction aFunction,TInt aStackSize,TAny* aPtr,RLibrary* aLibrary,RHeap* aHeap, TInt aHeapMinSize,TInt aHeapMaxSize,TOwnerType aType) {
		RABase::CloseRA();
		User::LeaveIfError(Create(aName,aFunction,aStackSize,aPtr,aLibrary,aHeap,aHeapMinSize,aHeapMaxSize,aType));
		iOpen=ETrue;
		PutOnStackL();
	}
	inline void OpenLA(const TDesC& aFullName,TOwnerType aType=EOwnerProcess) {
		RABase::CloseRA();
		User::LeaveIfError(Open(aFullName,aType));
		iOpen=ETrue;
		PutOnStackL();
	}
	inline void OpenLA(const TFindThread& aFind,TOwnerType aType=EOwnerProcess) {
		RABase::CloseRA();
		User::LeaveIfError(Open(aFind,aType));
		iOpen=ETrue;
		PutOnStackL();
	}
	inline void OpenLA(TThreadId aID,TOwnerType aType=EOwnerProcess) {
		RABase::CloseRA();
		User::LeaveIfError(Open(aID,aType));
		iOpen=ETrue;
		PutOnStackL();
	}
	~RAThread() { RABase::CloseRA(); }
	void CloseInner() { RThread::Close(); }
private:
	void Close() { }
};

class RATimer : public RTimer, public RABase {
public:
	inline void CreateLocalLA() {
		RABase::CloseRA();
		User::LeaveIfError(CreateLocal());
		iOpen=ETrue;
		PutOnStackL();
	}
	~RATimer() { RABase::CloseRA(); }
	void CloseInner() { RTimer::Close(); }
private:
	void Close() { }
};

class RAProcess : public RProcess, public RABase {
public:
	inline void CreateLA(const TDesC& aFileName,const TDesC& aCommand,const TUidType& aUidType, TOwnerType aType=EOwnerProcess) {
		RABase::CloseRA();
		User::LeaveIfError(Create(aFileName,aCommand,aUidType,aType));
		iOpen=ETrue;
		PutOnStackL();
	}
	inline void CreateLA(const TDesC& aFileName,const TDesC& aCommand,TOwnerType aType=EOwnerProcess) {
		RABase::CloseRA();
		User::LeaveIfError(Create(aFileName,aCommand,aType));
		iOpen=ETrue;
		PutOnStackL();
	}
	inline void OpenLA(const TDesC& aName,TOwnerType aType=EOwnerProcess) {
		RABase::CloseRA();
		User::LeaveIfError(Open(aName,aType));
		iOpen=ETrue;
		PutOnStackL();
	}
	inline void OpenLA(const TFindProcess& aFind,TOwnerType aType=EOwnerProcess) {
		RABase::CloseRA();
		User::LeaveIfError(Open(aFind,aType));
		iOpen=ETrue;
		PutOnStackL();
	}
	inline void OpenLA(TProcessId aId,TOwnerType aType=EOwnerProcess) {
		RABase::CloseRA();
		User::LeaveIfError(Open(aId,aType));
		iOpen=ETrue;
		PutOnStackL();
	}
	~RAProcess() { RABase::CloseRA(); }
	void CloseInner() { RProcess::Close(); }
private:
	void Close() { }
};

#endif // CC_RAE32STD_H_INCLUDED
