#include "alerter.h"
#include "app_context.h"
#include "settings.h"
#include "symbian_auto_ptr.h"

#include <CFLDRingingTonePlayer.h>

#define SETTING_VIBRATE_ONLY	27
#include <eikenv.h>

#include <profileapi.h>
#ifndef NO_PROFILEAPI_H
#else
typedef TInt32 TContactItemId;
class CProfileAPI : public CBase {
public:
	enum TProErrorCode
	    {
	    EPro0=0
	   ,EPro1
	    };
	IMPORT_C static CProfileAPI* NewL(TBool);
	IMPORT_C virtual ~CProfileAPI();
	IMPORT_C TProErrorCode GetProfileActiveName(TPtr, TInt*);
	IMPORT_C TProErrorCode GetProfileMultiData( TDes&, TDes&, TInt&,
					       TInt&, TBool&, TInt&, CArrayFixFlat<TContactItemId>*,
					       TInt&, TInt);
};
#endif

class CAlerterImpl : public CAlerter, public MContextBase
{
private:
	CAlerterImpl(MApp_context& Context);
	void ConstructL();

	virtual void ShortAlert();
	virtual void AlertUser();
	virtual void StopAlert();
	~CAlerterImpl();

	virtual void PlayAlert(TBool aShort);

	int get_current_profile();
#ifndef NO_CFLDRINGINGTONEPLAYER_H
	CFLDRingingTonePlayer* iPlayer;
	bool	ringing;
#endif
	
	friend class CAlerter;
	friend class auto_ptr<CAlerterImpl>;
};

EXPORT_C CAlerter* CAlerter::NewL(MApp_context& Context)
{
	auto_ptr<CAlerterImpl> ret(new (ELeave) CAlerterImpl(Context));
	ret->ConstructL();
	return ret.release();
}

CAlerterImpl::CAlerterImpl(MApp_context& Context) : MContextBase(Context)
{
}

void CAlerterImpl::ConstructL()
{
#ifndef __WINS__
#ifndef NO_CFLDRINGINGTONEPLAYER_H
	iPlayer=CFLDRingingTonePlayer::NewL(ETrue);
	iPlayer->SetVibra(ETrue);
	iPlayer->SetRingingType(2); // ring once

	// We'll fool the ringtoneplayer about fg/bg events
	CEikonEnv::Static()->RemoveForegroundObserver(*iPlayer);
	((MCoeForegroundObserver*)iPlayer)->HandleGainingForeground();
#endif
#endif
}

int CAlerterImpl::get_current_profile()
{
#ifndef __WINS__
	auto_ptr<CProfileAPI> profile(CProfileAPI::NewL(EFalse));
	auto_ptr<HBufC> name(HBufC::NewL(30));
	TPtr16 p=name->Des();

	TInt id;
	if (profile->GetProfileActiveName(p, &id)!= 0)
		return -1; // Get active profile

	if (id<0 || id>10) return -1;

	return id;
#else
	return 0;
#endif
}

void CAlerterImpl::AlertUser()
{
	PlayAlert(EFalse);
}

void CAlerterImpl::ShortAlert()
{
	PlayAlert(ETrue);
}


void CAlerterImpl::PlayAlert(TBool aShort)
{
#ifndef __WINS__
#ifndef NO_CFLDRINGINGTONEPLAYER_H
	TFileName tone=DataDir();

	int profile=get_current_profile();

	switch (profile) {
	case 2:
		// meeting
		if (!aShort) {
			tone.Append(_L("silent.rng"));
		} else {
			tone.Append(_L("shortsilent.rng"));
		}
		break;
	case 1:
		return;
		// silent
		break;
	default:
		{
		// general, outdoor
		TBool only_vibrate=true;
		Settings().GetSettingL(SETTING_VIBRATE_ONLY, only_vibrate);
		if (only_vibrate) {
			if (!aShort) {
				tone.Append(_L("silent.rng"));
			} else {
				tone.Append(_L("shortsilent.rng"));
			}
		} else {
			if (!aShort) {
				tone.Append(_L("tone.rng"));
			} else {
				tone.Append(_L("shorttone.rng"));
			}
		}		
		break;
		}
	}

	ringing=true;
	((MFLDFileProcessor*)iPlayer)->ProcessFileL(tone);
#endif
#endif
}

void CAlerterImpl::StopAlert()
{
#ifndef __WINS__
#ifndef NO_CFLDRINGINGTONEPLAYER_H
	if (ringing) {
		((MFLDFileProcessor*)iPlayer)->Cancel();
		ringing=false;
	}
#endif
#endif
}

CAlerterImpl::~CAlerterImpl()
{
#ifndef __WINS__
#ifndef NO_CFLDRINGINGTONEPLAYER_H
	if (iPlayer) {
		((MCoeForegroundObserver*)iPlayer)->HandleLosingForeground();
		CEikonEnv::Static()->AddForegroundObserverL(*iPlayer);
	}
	delete iPlayer;
#endif
#endif
}
