#include "contextappui.h"
#include "list.h"
#include "file_logger.h"
#include "app_context_impl.h"
#include "cl_settings.h"
#include "reporting.h"
#include <eikenv.h>
#include <apgtask.h>
#include "break.h"
#include "callstack.h"

EXPORT_C void MContextAppUi::ConstructL(MApp_context& aContext, const TDesC& aLogFileName, TInt aMaxLogCount)
{
#ifdef __WINS__
	User::Check();
#endif
	iWait=CTimeOut::NewL(*this);
	iWaitingOps=CList<TCallBack>::NewL();
	StepDone();

	iAppContext=&aContext;
	iLog=Cfile_logger::NewL(aContext, aLogFileName, CBBSensorEvent::INFO, aMaxLogCount);
	aContext.SetFileLog(iLog);
	StepDone();
#ifdef __WINS__
	User::Check();
#endif
}

EXPORT_C TInt MContextAppUi::InitializationSteps()
{
	return 2;
}

EXPORT_C void MContextAppUi::StepDone()
{
}

EXPORT_C void MContextAppUi::ReleaseContextAppUi()
{
#ifdef __WINS__
	User::Check();
#endif
	delete iWait; iWait=0;
	delete iWaitingOps; iWaitingOps=0;
	delete iLog; iLog=0;
	if (GetContext()) GetContext()->SetFileLog(0);
#ifdef __WINS__
	User::Check();
#endif
}
EXPORT_C MContextAppUi::~MContextAppUi()
{
	ReleaseContextAppUi();
}

EXPORT_C void MContextAppUi::QueueOp(TCallBack Op, int AfterSeconds)
{
	CALLSTACKITEM_N(_CL("MContextAppUi"), _CL("QueueOp"));

	iWaitingOps->AppendL(Op);
	iWait->Wait(AfterSeconds);
}

EXPORT_C void MContextAppUi::RunOp(TCallBack op)
{
	((*this).*op)();
}

EXPORT_C void MContextAppUi::expired(CBase*)
{
	CALLSTACKITEM_N(_CL("CContextContactsAppUi"), _CL("expired"));

	TCallBack op=iWaitingOps->Pop();
	RunOp(op);
	while (iWaitingOps->iFirst && iWaitingOps->Top()==op) iWaitingOps->Pop();
	if (iWaitingOps->iFirst) {
		iWait->Wait(1);
	}
}

EXPORT_C void MContextAppUi::hide()
{
	CALLSTACKITEM_N(_CL("MContextAppUi"), _CL("hide"));

	RWsSession& wsSession=CEikonEnv::Static()->WsSession();
	
	TApaTask task(wsSession);
	TInt id = CEikonEnv::Static()->RootWin().Identifier();
	task.SetWgId(id);
		
	if ( wsSession.GetFocusWindowGroup() == id )
	{
		task.SendToBackground();
	}
}

EXPORT_C void MContextAppUi::unhide()
{
	CALLSTACKITEM_N(_CL("MContextAppUi"), _CL("unhide"));

	RWsSession& wsSession=CEikonEnv::Static()->WsSession();
	TApaTask task(wsSession);
	task.SetWgId(CEikonEnv::Static()->RootWin().Identifier());
	task.BringToForeground();
}

EXPORT_C void MContextAppUi::ReportError(const TDesC& Source,
	const TDesC& Reason, TInt Code)
{
	_LIT(err, "error");
	TBBLongString msg(err);
	msg()=_L("Unhandled error ");
	msg().AppendNum(Code);
	msg().Append(_L(" "));
	msg().Append(Reason.Left(120));
	if (iLog) {
		TInt err;
		CC_TRAP(err, iLog->new_value(CBBSensorEvent::ERR, _L("APP"), msg(), GetTime()));
	}
}

EXPORT_C void MContextAppUi::LogFormatted(const TDesC& aMsg)
{
	if (iLog) {
		CC_TRAPD(err, iLog->new_value(CBBSensorEvent::ERR, _L("APP"), aMsg, GetTime()));
	}
}


EXPORT_C TErrorHandlerResponse MContextAppUi::HandleError(TInt aError,
     const SExtendedError& aExtErr,
     TDes& aErrorText,
     TDes& aContextText)
{
	// TODO: handle aExtErr
	TInt err;
	if (iLog) {
		TBuf<128> msg;
		msg.Format(_L("Unhandled error %d %S %S"), aError, &aContextText, &aErrorText);
		TTime t=GetTime();
		CC_TRAP(err, iLog->new_value(CBBSensorEvent::ERR, _L("app_event"), msg, t));
#ifdef __WINS__
		RDebug::Print(msg);
#endif

		HBufC* stack=0;
		if (iAppContext) { 
			stack=iAppContext->CallStackMgr().GetFormattedCallStack(_L("AppUi"));
		}
		if (stack) {
			CC_TRAP(err, iLog->new_value(CBBSensorEvent::ERR, _L("app_event"), *stack, t));
		}
		delete stack;
		if (iAppContext) {
			iAppContext->CallStackMgr().ResetCallStack();
		}
	}

	if (!iHandlableError) {
		CEikonEnv::Static()->AppUi()->PrepareToExit();
		// const TInt KContextErrTimeoutInBBProtocol=-11001;
		if (aError==-11001) {
			// will notify starter to restart phone
			// if occurs more than once
			iExitValue=2004;
			
		} else {
			iExitValue=2003;
		}
		RDebug::Print(_L("delete this"));
		delete this;
		RDebug::Print(_L("User::Exit"));

		User::Exit(iExitValue);
		return ENoDisplay;
	} else {
		return EAlertDisplay;
	}
}

EXPORT_C void MContextAppUi::SetInHandlableEvent(TBool aHandlable)
{
	iHandlableError=aHandlable;
}

