#include "csd_lookup.h"

EXPORT_C const TTypeName& CBBLookup::Type() const
{
	return KLookupType;
}

EXPORT_C const TTypeName& CBBLookup::StaticType()
{
	return KLookupType;
}

EXPORT_C TBool CBBLookup::Equals(const MBBData* aRhs) const
{
	const CBBLookup* rhs=bb_cast<CBBLookup>(aRhs);
	return (rhs && *rhs==*this);
}

EXPORT_C CBBLookup& CBBLookup::operator=(const CBBLookup& aInfo)
{
	iLooker()=aInfo.iLooker();
	iWhen()=aInfo.iWhen();
	*iPresence=*(aInfo.iPresence);
	return *this;
}

EXPORT_C MBBData* CBBLookup::CloneL(const TDesC& ) const
{
	auto_ptr<CBBLookup> ret(new (ELeave) CBBLookup(2));
	ret->ConstructL();
	*ret=*this;
	return ret.release();
}

const MBBData* CBBLookup::Part(TUint aPartNo) const
{
	switch(aPartNo) {
	case 0:
		return &iLooker;
	case 1:
		return &iLooked;
	case 2:
		return &iWhen;
	case 3:
		return iPresence;
	case 4:
		if (iUseVersion==1) return 0;
		else return &iPresenceTimeStamp;
	default:
		return 0;
	}
}

EXPORT_C CBBLookup* CBBLookup::NewL()
{
	return NewL(2);
}

EXPORT_C CBBLookup* CBBLookup::NewL(TUint aVersion)
{
	auto_ptr<CBBLookup> ret(new (ELeave) CBBLookup(aVersion));
	ret->ConstructL();
	return ret.release();
}

EXPORT_C CBBLookup::~CBBLookup()
{
	delete iPresence;
}

EXPORT_C bool CBBLookup::operator==(const CBBLookup& aRhs) const
{
	return (
		iLooker==aRhs.iLooker &&
		iLooked==aRhs.iLooked &&
		iWhen==aRhs.iWhen &&
		*iPresence==*(aRhs.iPresence)
		);
}

_LIT(KSpace, " ");
EXPORT_C const TDesC& CBBLookup::StringSep(TUint ) const
{
	return KSpace;
}

EXPORT_C CBBLookup::CBBLookup(TUint aVersion) : TBBCompoundData(KLookup), iLooker(KLooker), 
	iLooked(KLooked), iWhen(KWhen), iPresenceTimeStamp(KTimeStamp), iCreatedVersion(aVersion),
	iUseVersion(2) { }

EXPORT_C void CBBLookup::ConstructL()
{
	iPresence=CBBPresence::NewL();
}

EXPORT_C void CBBLookup::InternalizeL(RReadStream& aStream)
{
	iUseVersion=iCreatedVersion;
	TBBCompoundData::InternalizeL(aStream);
	iUseVersion=2;
}
