/* 
    Copyright (C) 2004  Mika Raento - Renaud Petit

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA


    email: mraento@cs.helsinki.fi - petit@cs.helsinki.fi 
*/


#include "PresenceInfo.h"
#include "ContextServer.h"

#include "SocketsEngine.h"

#include <flogger.h>
#include "app_context.h"
#include "csd_presence.h"
#include "break.h"
#include "bbxml.h"

CPresenceInfo * CPresenceInfo::NewL()
{
	CALLSTACKITEMSTATIC_N(_CL("CPresenceInfo"), _CL("NewL"));

	CPresenceInfo * presenceInfo = CPresenceInfo::NewLC();
    	CleanupStack::Pop(presenceInfo);
    	return presenceInfo;
}

CPresenceInfo * CPresenceInfo::NewLC()
{
	CALLSTACKITEMSTATIC_N(_CL("CPresenceInfo"), _CL("NewLC"));

	CPresenceInfo* presenceInfo = new (ELeave) CPresenceInfo();
    	CleanupStack::PushL(presenceInfo);
    	presenceInfo->ConstructL();
    	return presenceInfo;
}

CPresenceInfo::~CPresenceInfo()
{
	CALLSTACKITEM_N(_CL("CPresenceInfo"), _CL("~CPresenceInfo"));

	delete iBBSession;
	delete iAllContactsArray;
	delete iAllContactsPresenceInfoArray;
	delete iFlagList;
	if (iSend_timestamps) iSend_timestamps->Close();
	delete iSend_timestamps;

}

void CPresenceInfo::ConstructL()
{
	CALLSTACKITEM_N(_CL("CPresenceInfo"), _CL("ConstructL"));

	iBBSession=BBSession()->CreateSubSessionL(this);
	iBBSession->AddNotificationL(KPresenceFrozen);

	MBBData* fr=0; TInt err;
	CC_TRAP(err, iBBSession->GetL(KPresenceFrozen, KNullDesC, fr));
	if (err==KErrNone) {
		const TBBBool* frozen=bb_cast<TBBBool>(fr);
		if (frozen && (*frozen)()) { iFrozen=ETrue; }
	}
	delete fr;

	iAllContactsArray             = new (ELeave) CDesCArrayFlat(KMaxContacts);
	iAllContactsPresenceInfoArray = new (ELeave) CDesCArrayFlat(KMaxContacts);
	iFlagList = CList< RArray<TInt>* >::NewL();
	iSend_timestamps = new (ELeave) RArray<TTime>;
}

void CPresenceInfo::RemoveFlag(TInt pos)
{
	CList< RArray<TInt> * >::Node *n=iFlagList->iFirst;
	while (n) {
		n->Item->Remove(pos);
		n=n->Next;
	}
}

void CPresenceInfo::AppendFlag(TInt val)
{
	CList< RArray<TInt> * >::Node *n=iFlagList->iFirst;
	while (n) {
		User::LeaveIfError(n->Item->Append(val));
		n=n->Next;
	}
}

void CPresenceInfo::CompressFlag()
{
	CList< RArray<TInt> * >::Node *n=iFlagList->iFirst;
	while (n) {
		n->Item->Compress();
		n=n->Next;
	}
}

void CPresenceInfo::NewValueL(TUint aId, const TTupleName& aName, const TDesC& aSubName, 
	const TComponentName& aComponentName, const MBBData* aData) 
{
	if (aName==KPresenceFrozen) {
		const TBBBool* frozen=bb_cast<TBBBool>(aData);
		if (frozen && (*frozen)()) {
			iFrozen=ETrue;
		} else {
			iFrozen=EFalse;
		}
	}
}

void CPresenceInfo::DeletedL(const TTupleName& aName, const TDesC& aSubName)
{
	if (aName==KPresenceFrozen) {
		iFrozen=EFalse;
	}
}

void CPresenceInfo::Update(const TDesC & contact, const TDesC& presence, const TTime & time)
{
	CALLSTACKITEM_N(_CL("CPresenceInfo"), _CL("Update"));

	TInt pos;

	if (!iFrozen) {
		TBuf<15> pv2=_L("<presencev2");
		if (presence.Left(pv2.Length()).Compare(pv2)) goto skip;

		TPtrC8 xml8( (TUint8*)presence.Ptr(), presence.Size());

		auto_ptr<CBBPresence> data( CBBPresence::NewL() );
		auto_ptr<CSingleParser> parser( CSingleParser::NewL(data.get(), EFalse, ETrue) );
		CC_TRAPD(err, parser->ParseL(xml8) );
		if (err!=KErrNone) goto skip;
		data->iSentTimeStamp()=time;

		TTime expires; expires.HomeTime(); expires+=TTimeIntervalDays(5);
		iBBSession->PutL(KIncomingPresence, contact, data.get(), expires);
	}
skip:
	if (iAllContactsArray->Find(contact ,pos) == 0)
	// found
	{
		iAllContactsPresenceInfoArray->Delete(pos);
		iAllContactsArray->Delete(pos);

		RemoveFlag(pos);
		
		iSend_timestamps->Remove(pos);

		iAllContactsArray->Compress();
		iAllContactsPresenceInfoArray->Compress();
		CompressFlag();
		iSend_timestamps->Compress();
	}
		
	iAllContactsPresenceInfoArray->AppendL(presence);
	iAllContactsArray->AppendL(contact);
	AppendFlag(1);
	iSend_timestamps->Append(time);
}

const TPresenceInfo& CPresenceInfo::GetNewPresenceInfo(RArray<TInt>* Flags)
{
	CALLSTACKITEM_N(_CL("CPresenceInfo"), _CL("GetNewPresenceInfo"));

	TInt pos = Flags->Find(1);

	info.newInfo = 0;

	if ( pos != KErrNotFound)
	{
		(*Flags)[pos] = 0;
		info.contact.Set( (*iAllContactsArray)[pos] );
		info.info.Set( (*iAllContactsPresenceInfoArray)[pos] );
		info.send_timestamp = (*iSend_timestamps)[pos];
		info.newInfo = 1;
	}
	return info;
}

void CPresenceInfo::SetAllAsNew(RArray<TInt>* Flags)
{
	CALLSTACKITEM_N(_CL("CPresenceInfo"), _CL("SetAllAsNew"));

	for (int i=0;i<Flags->Count();i++)
	{
		(*Flags)[i] = 1;
	}
}

void CPresenceInfo::SetAsNew(const TDesC & contact, RArray<TInt>* Flags)
{
	CALLSTACKITEM_N(_CL("CPresenceInfo"), _CL("SetAsNew"));

	TInt pos;
	if (iAllContactsArray->Find(contact ,pos) == 0)
	{
		(*Flags)[pos] = 1;
	}
}


TPtrC CPresenceInfo::GetPresenceInfoAsDes()
{
	CALLSTACKITEM_N(_CL("CPresenceInfo"), _CL("GetPresenceInfoAsDes"));

	TBuf<800> info ;

	for (int i = 0;i<iAllContactsArray->Count();i++)
	{
		info.Append((*iAllContactsArray)[i]);
		info.Append(_L(":"));
		info.Append((*iAllContactsPresenceInfoArray)[i]);
		info.Append(_L(";"));
	}
	return TPtrC(info);
}

	
void CPresenceInfo::RemoveFlags( RArray<TInt>* Flags)
{
	CList< RArray<TInt>* >::Node *n=iFlagList->iFirst;
	do {
		if (n->Item == Flags) {
			iFlagList->DeleteNode(n, true);
			return;
		}
	} while ( (n=n->Next) );
}

void CPresenceInfo::AddFlagsL( RArray<TInt>* Flags)
{
	iFlagList->AppendL(Flags);
	for (int i=0; i<iAllContactsArray->Count(); i++) {
		User::LeaveIfError(Flags->Append(1));
	}
}
