#ifndef CONTEXT_APPUIBASE_H_INCLUDED
#define CONTEXT_APPUIBASE_H_INCLUDED 1

#include <aknviewappui.h>
class CAppLogView;
class CBTDeviceList;
class CTransferDir;
class CPicturePublisher;
#include "reporting.h"
#include "sensorrunner.h"
#include "status_notif.h"
#include "app_context.h"
#include "contextappui.h"
#include "connectioninit.h"
#include "locationing.h"
#include "timeout.h"
#include "cm_autotaglist.h"
#include "ntpconnection.h"
#include <AknProgressDialog.h>

class CContextLogAppUiBase : public CAknViewAppUi, public i_status_notif,
	public MContextBase, public MSocketObserver, 
	public MNaming,
	public MContextAppUi, public MAppEvents, public MAskForNames,
	public MNTPObserver, public MProgressDialogCallback
{
protected:
	CContextLogAppUiBase(MApp_context& Context) : MContextBase(Context) { initialising=true; }
        void ConstructL();
	void ConstructAfterPresenceMaintainerL();
	void FinalConstructL();
        ~CContextLogAppUiBase();
	static TInt InitializationSteps();

	bool initialising;
	// i_Clog_comm_notif
	void finished();
	void error(const TDesC& descr);
	void status_change(const TDesC& status);
	
	// MNTPObserver
	virtual void NTPInfo(const TDesC& aMsg);
	virtual void NTPError(const TDesC& aMsg, TInt aErrorCode);
	virtual void NTPSuccess(TTime aNewTime);

	// MSocketObserver
	virtual void success(CBase* source) {
		if (source==iLastErrorFrom) {
			iLastErrorFrom=0;
			error(_L(""));
		}
	}
	virtual void error(CBase* source, TInt code, const TDesC& reason);
	virtual void info(CBase* source, const TDesC& msg) {
		if (source==iLastErrorFrom) {
			iLastErrorFrom=0;
			error(_L(""));
		}
		status_change(msg);
	}
        TBool BaseHandleCommandL(TInt aCommand); // did handle?
	virtual TErrorHandlerResponse HandleError
	    (TInt aError,
	     const SExtendedError& aExtErr,
	     TDes& aErrorText,
	     TDes& aContextText);
	virtual void LogFormatted(const TDesC& aMsg);
	void LogAppEvent(const TDesC& msg);
	virtual void HandleSystemEventL  (  const TWsEvent &    aEvent  );
	virtual void HandleWsEventL(const TWsEvent& aEvent, CCoeControl* aDestination);
	void AddMediaPublisher(CPicturePublisher* aPublisher);
	virtual TBool ProcessCommandParametersL(TApaCommand aCommand, 
		TFileName& aDocumentName,const TDesC8& aTail);
	void PublishOldMedia();
	void expired(CBase* aSource);

	void add_cellid_name(const TBBCellId& cellid, const TDesC& name);

	virtual TBool NameCell(TInt aBaseId, const class TBBCellId& aCellId, TDes& aNameInto);
	virtual TBool NameCity(const class TBBCellId& aCellId, TDes& aNameInto);
	virtual TBool GetExistingCell(TInt aBaseId, const class TBBCellId& aCellId, TDes& aNameInto);
	virtual TBool GetExistingCity(const class TBBCellId& aCellId, TDes& aNameInto);

	virtual void  DialogDismissedL (TInt aButtonId);

	class CContext_logContainer* iAppContainer; 
	class CStatusView*		iStatusView;
	class Mlog_base_impl *app_log;
	
	class sms * smsh;
	CBase		*iLastErrorFrom;
	class CCircularLog	*iLastLog;
	CArrayPtrFlat<class CPicturePublisher> *iMediaPublishers;

	TBuf<30> state;
	class CAppLogView		*iLogView; // OWNED BY FRAMEWORK
	CArrayPtrFlat<class Mlog_base_impl>	*iLoggers;
	class CDb			*iBTDb;
	class CBTDeviceList		*iBuddyBTs, *iLaptopBTs, *iDesktopBTs, *iPDABTs;
	class CTransferDir		*iTransferDir;
	class MUploadPrompt	*iOldPrompt;
	class CTimeOut		*iWatchDogTimer;
	class CNTPConnection	*iNTPConnection;
	class CSensorRunner	*iSensorRunner;
	class CDb			*iMediaDb;
	class CPostStorage	*iMediaStorage;
	class CMediaPrompt	*iMediaPrompt;
	class CBBSubSession	*iBBSubSession;
	class CCellNaming	*iCellNaming;
	class CAknWaitDialog	*iNTPWait;

	class MPresenceMaintainer *iPresenceMaintainer; //owned by derived class
	class MNetworkConnection *iNetworkConnection; //owned by derived class

	const TBBCellId*	iAskedFor; TDes* iAskedInto; TBool iGotAsked;
	TInt iNtpErrors;
public:
	void start_app(TUid app, TInt view);
};

#endif
