/* 
    Copyright (C) 2004  Mika Raento - Renaud Petit

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA


    email: mraento@cs.helsinki.fi - petit@cs.helsinki.fi 
*/


#pragma warning(disable: 4706)

// INCLUDE FILES
#include "Context_logAppUi.h"
#include <context_log.rsg>
#include <sendnorm.rsg>
#include "local_defaults.h"
#include "context_log.hrh"
#include "context_logapp.h"

#define CONTEXTNW 1
//#undef CONTEXTNW
//#undef LOCA

#include <avkon.hrh>
#include <aknquerydialog.h> 
#include <eikenv.h> // ieikonenv

#include <eikmenup.h>
#include <apgcli.h>

#include <bautils.h>

#include <RPbkViewResourceFile.h>
#include <CPbkContactEditorDlg.h>
#include <CPbkContactEngine.h>
#include <CPbkSelectFieldDlg.h>
#include <CPbkContactItem.h>

#include <apgwgnam.h>

#include "userview.h"
#include "presencedetailview.h"
#include "presencedescriptionview.h"

#include "cl_settings.h"

#include "break.h"

#include "hideview.h"

#include "notifystate.h"
#include <context_log.mbg>
#include "viewids.h"

#include "raii_apgcli.h"
#include "raii_f32file.h"

#include "loca_sender.h"
#include "sms_status.h"
#include "callstack.h"
#include "reporting.h"
#include "app_context_impl.h"
#include "presence_ui_helper.h"

#include "contextnotifyclientsession.h"

#include "csd_sms.h"

#include "dumper.h"
#include "bbl_runner.h"

#include "cu_errorui.h"
#include "emptytext.h"
#include "file_logger.h"
#include "csd_battery.h"
#include "csd_system.h"
#include "csd_current_app.h"
#include "bt_dev_ap_view.h"
#include "log_comm.h"
#include "transfer.h"
#include "context_logcontainer.h"
#include "call_recorder.h"
#include <eikdll.h>
#include "mediaprompt.h"
#include "ntpconnection.h"
#include "cellnaming.h"
#include "settingsview.h"
#include "sms_snapshot.h"

_LIT(cellid_file, "cellid_names_v2.txt");
_LIT(transfer_cellid_file, "cellid_names_trans_v2.txt");

//#include <syevdef.h>

enum JOYSTICK_EVENTS {
	JOY_LEFT = 0xF807,
	JOY_RIGHT = 0xF808,
	JOY_UP = 0xF809,
	JOY_DOWN = 0xF80A,
	JOY_CLICK = 0xF845
};

const TUid KUidPhone = { 0x100058b3 };
const TUid KUidMenu = {  0x101f4cd2 };

#ifndef __WINS__
_LIT(KIconFile, "c:\\system\\data\\context_log.mbm");
#else
_LIT(KIconFile, "z:\\system\\data\\context_log.mbm");
#endif

#ifdef __WINS__
class TDummyMsvObserver : public MMsvSessionObserver
{
	void HandleSessionEvent(TMsvSessionEvent, TAny*, TAny*, TAny*) { }
	void HandleSessionEventL(TMsvSessionEvent, TAny*, TAny*, TAny*) { }
};
#endif


void CContext_logAppUi::ConstructL()
{
	CALLSTACKITEM_N(_CL("CContext_logAppUi"), _CL("ConstructL"));

#ifndef __WINS__
	bool wins=false;
#else
	bool wins=true;
#endif

	CContextLogAppUiBase::ConstructL();

#ifndef DONT_LOG_EVENTS_TO_FILE

	iLog->SubscribeL(KCellIdTuple);
	iLog->SubscribeL(KBaseTuple);
	iLog->SubscribeL(KProfileTuple);
	iLog->SubscribeL(KBluetoothTuple);
	iLog->SubscribeL(KOwnBluetoothTuple);
	iLog->SubscribeL(KGpsTuple);
	iLog->SubscribeL(KAlarmTuple);
	iLog->SubscribeL(KUnreadTuple);
	iLog->SubscribeL(KCalendarTuple);
	iLog->SubscribeL(KIdleTuple);
	iLog->SubscribeL(KBatteryTuple);
	iLog->SubscribeL(KNetworkTuple);
	iLog->SubscribeL(KChargerTuple);
	iLog->SubscribeL(KCurrentAppTuple);
	iLog->SubscribeL(KAppEventTuple);
	iLog->SubscribeL(KSmsTuple);
	iLog->SubscribeL(KUserGivenContextTuple);

#else
	// we always want errors to be logged
	iLog->SubscribeL(KAppEventTuple);
#endif

#if CONTEXTNW
	state=_L("bbl");
	bbl=CBBLogger::NewL(AppContext(),this);
#endif

	iBBLocal=CBBLocalRunner::NewL(AppContext(), this);
	//iSocketTestReader=CSocketTestReader::NewL(AppContext(), 2000, _L("c:\\bblocal.xml"));

	{
		TBool logging_enabled;
		Settings().GetSettingL(SETTING_LOGGING_ENABLE, logging_enabled);
		TInt notif_err;
		CC_TRAP(notif_err, iLoggingRunning=CNotifyState::NewL(AppContext(), KIconFile));
		if (logging_enabled) {
			if(iLoggingRunning) iLoggingRunning->SetCurrentState( EMbmContext_logL, EMbmContext_logL );
		} else {
			if(iLoggingRunning) iLoggingRunning->SetCurrentState( EMbmContext_logL_not, EMbmContext_logL_not );
		}
	}

#ifdef LOCA
	state=_L("LocaSender");
	iLocaSender=CLocaSender::NewL(AppContext());
#endif

	ringing=false;

	iCaptionMyContext=CEikonEnv::Static()->AllocReadResourceL(R_ME_CAPTION);
	
	state=_L("notify");

	Settings().NotifyOnChange(SETTING_LOGGING_ENABLE, this);

	//__UHEAP_MARK;

#if DO_SMS_STATUS
	state=_L("sms status");
	if (smsh) iSmsStatus=CSmsStatusReplier::NewL(AppContext(), smsh);
#endif


#ifndef NO_PRESENCE
	state=_L("iUserContextLog");

	iUserContextLog=CCircularLog::NewL(1,ETrue);
	
	CEikonEnv::Static()->ReadResourceAsDes16(prev, R_PREVIOUS);
	CEikonEnv::Static()->ReadResourceAsDes16(not_avail, R_JABBER_NOT_AVAIL);

	iUserContext=HBufC::NewL(iCaptionMyContext->Length()+256);
	PresenceToListBoxL(0, iUserContext, iCaptionMyContext, 0, KNullDesC, GetTime(),
		prev, not_avail);
	iUserContextLog->AddL(*iUserContext);

#endif


#ifndef NO_PRESENCE
	state=_L("create presence publisher");
	iPresenceMaintainer=iPresencePublisher=CPresencePublisher::NewL(AppContext(), this, *this, 
		iBuddyBTs, iLaptopBTs, iDesktopBTs, iPDABTs, this);
	iNetworkConnection=iPresencePublisher;
	LogAppEvent(_L("Starting 2.1"));
	RestoreUserGiven();
	Settings().NotifyOnChange(SETTING_OWN_DESCRIPTION, this);
	LogAppEvent(_L("Starting 2.2"));
#endif
	ConstructAfterPresenceMaintainerL();

	TBool enable_options=false;

#ifndef NO_PRESENCE
	state=_L("create user view");
	auto_ptr<CUserView> userview(CUserView::NewL(iUserContextLog, iPresencePublisher));
	AddViewL(userview.get());
	CUserView* v=userview.release();
	iUserView=v;
#  ifdef REALLY_ONLY_LOGGING
	SetDefaultViewL(*v);
#  else
	if ( ! Settings().GetSettingL(SETTING_OPTIONS_ENABLE, enable_options) || !enable_options) {
		SetDefaultViewL(*v);
	}
#  endif
#endif // NO_PRESENCE

#ifdef NO_PRESENCE
	state=_L("create hide view");
	auto_ptr<CHideView> Hideview(CHideView::NewL());
	AddViewL(Hideview.get()); 
	if ( Settings().GetSettingL(SETTING_OPTIONS_ENABLE, enable_options) && enable_options) {
		Hideview.release();
	} else {
		SetDefaultViewL(*(Hideview.release()));
	}
#endif


#ifndef NO_PRESENCE
	state=_L("create presence detail");
	auto_ptr<CPresenceDetailView> detailview(CPresenceDetailView::NewL());
	AddViewL(detailview.get());
	iPresenceDetailView=detailview.release();

	state=_L("create presence description");
	auto_ptr<CPresenceDescriptionView> Descriptionview(CPresenceDescriptionView::NewL());
	AddViewL(Descriptionview.get());
	iPresenceDescriptionView=Descriptionview.release();

	auto_ptr<CBTDevApView> btView2(CBTDevApView::NewL(AppContext(), iBuddyBTs, EFalse));
	AddViewL(btView2.get());
	iBTBuddiesView = btView2.release();
#endif //NO_PRESENCE

	state=_L("create commlog");
	comml=Clog_comm::NewL(AppContext(), this);

	if (wins) 
		iAppContainer->SubscribeL(KIdleTuple);

	state=_L("create transferer");
	transferer=CSendUITransfer::NewL(AppContext(), this, 
		Econtext_logCmdSendAppUi, DataDir(), AppDir());
	transferer->add_filesL(_L("log*txt"), true);
	transferer->add_filesL(_L("starter*txt"), true);
	transferer->add_filesL(_L("book*txt"), true);
	transferer->add_filesL(_L("calllog*txt"), true);
	transferer->add_filesL(_L("comm*txt"), false);
	transferer->add_filesL(_L("rec*amr"), false);
	transferer->add_filesL(_L("cellid_names_trans.txt"), false);
	transferer->add_filesL(_L("mms*.*"), false);

#ifndef NO_PERIODIC_TRANSFER
	iPeriodicTransfer=CPeriodicTransfer::NewL(AppContext(), 24, this, iTransferDir);
#endif

	iSmsSnapshot=CSmsSnapshot::NewL(AppContext(), this, iPresenceMaintainer, 
		iTransferDir->Transferer());
	/*
	answerer=Canswering::NewL(AppContext(), this);
	lp->add_sinkL(answerer);
	iAppContainer->CurrentLoc()->add_sinkL(answerer);
	*/


#ifndef NO_CALLRECORDING
	state=_L("create call recorders");
	in_recorder=CCall_record::NewL(AppContext(), this, CCall_record::INCOMING);
	out_recorder=CCall_record::NewL(AppContext(), this, CCall_record::OUTGOING);
#endif
	state=_L("create settings view");

	{
		auto_ptr<CSettingsView> iSettingsView(CSettingsView::NewL(KSettingsViewId, AppContext(), 
			TLocalAppSettings::GetEnabledSettingsArray()));
		AddViewL(iSettingsView.get());
		iSettingsView.release();
	}

	FinalConstructL();

}

class CMediaTest : public CActive, MContextBase, MUploadCallBack {
public:
	TInt iCount;
	MUploadPrompt* iPrompt;
	CMediaTest(MApp_context& aContextBase, MUploadPrompt* pr) : CActive(CActive::EPriorityIdle),
		MContextBase(aContextBase), iPrompt(pr), iCount(392) { }
	void ConstructL() {
		CActiveScheduler::Add(this);
		Async();
		pic=_L("c:\\media\\pic.jpg");
	}
	~CMediaTest() {
		Cancel();
	}
	void Async() {
		if (IsActive()) return;
		TRequestStatus *s=&iStatus;
		User::RequestComplete(s, KErrNone);
		SetActive();
	}
	void RunPrompt() {
		User::__DbgSetAllocFail(RHeap::EUser, RHeap::EDeterministic, iCount);
		iPrompt->Prompt(fn, this);
	}
	void RunL() {
		User::__DbgSetAllocFail(RHeap::EUser, RHeap::ENone, 1);
		TBuf<100> msg=_L("CMediaTest, round: ");
		iCount++;
		msg.AppendNum(iCount);
		if (iCount==66) {
			TInt x;
			x=0;
		}
		Reporting().DebugLog(msg);
		fn=_L("c:\\media\\pic"); fn.AppendNum(iCount); fn.Append(_L(".jpg"));
		User::LeaveIfError(BaflUtils::CopyFile(Fs(), pic, fn));
		CC_TRAPD(err, RunPrompt());
		msg=_L("CMediaTest, round: " );
		msg.AppendNum(iCount);
		if (err!=KErrNone) {
			msg.Append(_L("got error from prompt: "));
			msg.AppendNum(err);
			//Async();
		} else {
			msg.Append(_L("success in prompt"));
		}
		Reporting().DebugLog(msg);
	}
	TFileName fn, pic;
	TFileOpStatus st;
	virtual TFileOpStatus Back(bool Upload, bool DeleteFromPhone, MBBData* Packet) {
		st.fn=fn;
		st.err=KErrNone;
		TBuf<100> msg=_L("CMediaTest, round: ");
		iCount++;
		msg.Append(_L(" got to Back"));
		Reporting().DebugLog(msg);
		Async();
		return st;
	}
	void DoCancel() { }
};

// ----------------------------------------------------
// CContext_logAppUi::~CContext_logAppUi()
// Destructor
// Frees reserved resources
// ----------------------------------------------------
//
CContext_logAppUi::~CContext_logAppUi()
{
	CALLSTACKITEM_N(_CL("CContext_logAppUi"), _CL("~CContext_logAppUi"));
	TInt err;

	{
		CLocalNotifyWindow::Destroy();

		delete iMediaTest;
		delete iDumper;

		TTime t2;
		t2=GetTime();
		if (iLog) 
			CC_TRAP(err, iLog->new_value(CBBSensorEvent::VALUE, _L("APP close"), state, t2));


		Settings().CancelNotifyOnChange(SETTING_LOGGING_ENABLE, this);
		Settings().CancelNotifyOnChange(SETTING_OWN_DESCRIPTION, this);

		if (iTestFileOpen) iTestFile.Close();

		delete comml;

		delete iCaptionMyContext;
		
		//delete lt;
		if (t) {
			t->Kill(0);
			delete t;
		}

	}


	{
		TTime t; t=GetTime();
		if (iLog) CC_TRAP(err,
			iLog->new_value(CBBSensorEvent::VALUE, _L("APP"), _L("3"), t));
		delete iPeriodicTransfer;
		delete transferer;
	}

	{
		delete iPresencePublisher;
		delete iUserContextLog;
		delete iUserContext;
	}

	TTime t=GetTime();
	{

		if (iLog) CC_TRAP(err,
			iLog->new_value(CBBSensorEvent::VALUE, _L("APP"), state, t));
	}
	{
		//delete answerer;
		delete in_recorder;
		delete out_recorder;
	}


	{
		CALLSTACKITEM_N(_CL("CContext_logAppUi"), _CL("~CContext_logAppUi_d1"));
		delete iSmsStatus;
	}
	{
		CALLSTACKITEM_N(_CL("CContext_logAppUi"), _CL("~CContext_logAppUi_d2"));
		delete iBBLocal;
	}
	{
		CALLSTACKITEM_N(_CL("CContext_logAppUi"), _CL("~CContext_logAppUi_d3"));
		delete iSocketTestReader;
	}
	{
		CALLSTACKITEM_N(_CL("CContext_logAppUi"), _CL("~CContext_logAppUi_d4"));
		delete bbl;
	}

	{
		CALLSTACKITEM_N(_CL("CContext_logAppUi"), _CL("~CContext_logAppUi_d6"));
		delete iContextRunning;
	}
	{
		CALLSTACKITEM_N(_CL("CContext_logAppUi"), _CL("~CContext_logAppUi_d5"));
		delete iLoggingRunning;
	}

	{
		CALLSTACKITEM_N(_CL("CContext_logAppUi"), _CL("~CContext_logAppUi_d7"));
		// near the end, since the cancelling doesn't work
		// right on s60v1 :-(
		delete iLocaSender;
	}

}


// ------------------------------------------------------------------------------
// CContext_logAppUi::::DynInitMenuPaneL(TInt aResourceId,CEikMenuPane* aMenuPane)
//  This function is called by the EIKON framework just before it displays
//  a menu pane. Its default implementation is empty, and by overriding it,
//  the application can set the state of menu items dynamically according
//  to the state of application data.
// ------------------------------------------------------------------------------
//
void CContext_logAppUi::DynInitMenuPaneL(
					 TInt aResourceId,CEikMenuPane* aMenuPane)
{
	CALLSTACKITEM_N(_CL("CContext_logAppUi"), _CL("DynInitMenuPaneL"));

	switch(aResourceId) {
	case R_OTHERS_MENU:
#ifndef REALLY_ONLY_LOGGING
		if (iLog) {
			if (iLog->is_paused()) {
				aMenuPane->SetItemDimmed(Econtext_logCmdAppPauseLog, ETrue);	
				aMenuPane->SetItemDimmed(Econtext_logCmdAppUnPauseLog, EFalse);
			} else {
				aMenuPane->SetItemDimmed(Econtext_logCmdAppPauseLog, EFalse);	
				aMenuPane->SetItemDimmed(Econtext_logCmdAppUnPauseLog, ETrue);
			}
		} else {
			aMenuPane->SetItemDimmed(Econtext_logCmdAppPauseLog, ETrue);	
			aMenuPane->SetItemDimmed(Econtext_logCmdAppUnPauseLog, ETrue);
		}
#endif
		break;

	case R_USER_ACTIONS:

#ifndef REALLY_ONLY_LOGGING
		if (iPresencePublisher) {
			if (iPresencePublisher->ConnectionSuspended()) {
				aMenuPane->SetItemDimmed(Econtext_logCmdAppSuspendPresence, ETrue);	
				aMenuPane->SetItemDimmed(Econtext_logCmdAppResumePresence, EFalse);
			} else {
				aMenuPane->SetItemDimmed(Econtext_logCmdAppSuspendPresence, EFalse);	
				aMenuPane->SetItemDimmed(Econtext_logCmdAppResumePresence, ETrue);
			}
		} else {
			aMenuPane->SetItemDimmed(Econtext_logCmdAppSuspendPresence, ETrue);	
			aMenuPane->SetItemDimmed(Econtext_logCmdAppResumePresence, ETrue);
		}
#endif
		break;
	case R_SENDUI_TOPMENU:
		transferer->DisplaySendMenuL(*aMenuPane, 1);
		break;
	case R_SENDUI_MENU:
		// sendui cascade
		transferer->DisplayMenuL(*aMenuPane);
		break;
	default:
		break;
	}
}



void CContext_logAppUi::HandleCommandL(TInt aCommand)
{
	CALLSTACKITEM_N(_CL("CContext_logAppUi"), _CL("HandleCommandL"));

	SetInHandlableEvent(ETrue);
#ifdef __WINS__
	TBreakItem b(GetContext());
#endif
	if (BaseHandleCommandL(aCommand)) return;

	switch ( aCommand )
	{
	case Econtext_logCmdContacts:
		{
			auto_ptr<CApaCommandLine> cmd(CApaCommandLine::NewL(_L("z:\\system\\apps\\phonebook\\phonebook.app")));
			cmd->SetCommandL(EApaCommandRun);
			CC_TRAPD(err, EikDll::StartAppL(*cmd));
			break;
		}
	case Econtext_logCmdLogs:
		{
			auto_ptr<CApaCommandLine> cmd(CApaCommandLine::NewL(_L("z:\\system\\apps\\logs\\logs.app")));
			cmd->SetCommandL(EApaCommandRun);
			CC_TRAPD(err, EikDll::StartAppL(*cmd));
			break;
		}
	case Econtext_logPresenceDetails:
		PresenceDetails();
		break;
	case Econtext_logPresenceDescription:
		PresenceDescription();
		break;

	case Econtext_logCmdAppPauseLog:
		Settings().WriteSettingL(SETTING_LOGGING_ENABLE, EFalse);
		break;
	case Econtext_logCmdAppUnPauseLog:
		Settings().WriteSettingL(SETTING_LOGGING_ENABLE, ETrue);
		break;
	case Econtext_logCmdAppGetCommLog:
		{
		if (comml->write_comm_log()) {
			_LIT(getting, "Getting comm log");
			iAppContainer->set_status(getting);
		} else {
			_LIT(no, "Nothing to do");
			iAppContainer->set_status(no);
		}
		break;
		}
	case Econtext_logCmdBTBuddies:
		if (iBTBuddiesView) {
			iBTBuddiesView->SetList(iBuddyBTs);
			ActivateLocalViewL(KBtDevSetViewId);
		}
		break;
	case Econtext_logCmdBTPDAs:
		if (iBTBuddiesView) {
			iBTBuddiesView->SetList(iPDABTs);
			ActivateLocalViewL(KBtDevSetViewId);
		}
		break;
	case Econtext_logCmdBTLaptops:
		if (iBTBuddiesView) {
			iBTBuddiesView->SetList(iLaptopBTs);
			ActivateLocalViewL(KBtDevSetViewId);
		}
		break;
	case Econtext_logCmdBTDesktops:
		if (iBTBuddiesView) {
			iBTBuddiesView->SetList(iDesktopBTs);
			ActivateLocalViewL(KBtDevSetViewId);
		}
		break;
	case Econtext_logCmdBT_Dev_AP:
		if (iBTDevApView)
			ActivateLocalViewL(KBtDevApViewId);
		break;
	case Econtext_logCmdCancelSend:
		// do nothing
		break;
	case Econtext_logCmdAppSettings:
		ActivateLocalViewL(KSettingsViewId);
		break;
	case Econtext_logCmdAppTest:
		{
		//loc->test();
		//recorder->test();
		DialogTest();
		}
		break;
	case Econtext_logCmdAppCrash:
		{
		TBuf<2> buf; buf.Append(_L("xxxx"));
		}
		break;
	case Econtext_logCmdAppImsi:
		{
			/*if (loc) status_change(loc->GetImsi());
			else */{
#ifndef __WINS__
				TPlpVariantMachineId machineId;
				PlpVariant::GetMachineIdL(machineId);
				status_change(machineId);
#else
				// Return a fake IMEI when working on emulator
				_LIT(KEmulatorImsi, "244050000000000");
				status_change(KEmulatorImsi);
#endif
			}
		}
			
		break;
	case Econtext_logMyContext:
		ActivateLocalViewL(KUserViewId);
		break;
	case Econtext_logCmdAppSuspendPresence:
		{
			LogAppEvent(_L("User suspended presence"));
			Settings().WriteSettingL(SETTING_PRESENCE_ENABLE, 0);
		}
		break;
	case Econtext_logCmdAppResumePresence:
		{
			LogAppEvent(_L("User resumed presence"));
			Settings().WriteSettingL(SETTING_PRESENCE_ENABLE, 1);
		}
		break;
	case Econtext_logCmdSetUserGiven:
		{
			SetUserGiven();
		}
		break;
	case Econtext_logCmdFreezePresence:
		LogAppEvent(_L("FreezePresence"));
		if (iPresencePublisher) iPresencePublisher->FreezeL();
		if (iUserView) iUserView->SetFrozen(ETrue);
		break;
	case Econtext_logCmdUnFreezePresence:
		LogAppEvent(_L("UnFreezePresence"));
		if (iPresencePublisher) iPresencePublisher->UnFreezeL();
		if (iUserView) iUserView->SetFrozen(EFalse);
		break;
	default:
		if (aCommand>Econtext_logCmdSendAppUi || aCommand==Econtext_logCmdSendFtp) {
			
			if (aCommand==Econtext_logCmdSendFtp) {
				//iPeriodicTransfer->Transfer(false);
				iPeriodicTransfer->Transfer(true);
			} else {
				status_change(_L("trying transfer"));

				/*
				cellid_name_file.Close();
				*/
				
				TFileName transfer_cellid_filen, cellid_filen;
				transfer_cellid_filen.Format(_L("%S%S"), &AppDir(), &transfer_cellid_file);
				cellid_filen.Format(_L("%S%S"), &DataDir(), &cellid_file);

				TInt ferr=BaflUtils::CopyFile(Fs(), cellid_filen, transfer_cellid_filen);
				if (ferr!=KErrNone) {
					TBuf<30> errmsg;
					errmsg.Format(_L("copy: %d"), ferr);
					error(errmsg);
					return;
				}
				/*
				ferr=cellid_name_file.Open(Fs(), cellid_filen, 
					EFileShareAny | EFileStreamText | EFileRead | EFileWrite);
				if (ferr!=KErrNone) {
					TBuf<30> errmsg;
					errmsg.Format(_L("reopen: %d, RESTART NOW"), ferr);
					error(errmsg);
					return;
				}
				*/
				iLog->switch_file();

				transferer->transfer_files(aCommand);
			}
			
		}
		break;
	}
}


_LIT(KUserGiven, "user given");

void CContext_logAppUi::RestoreUserGiven()
{
	CALLSTACKITEM_N(_CL("CContext_logAppUi"), _CL("RestoreUserGiven"));
	
	const TInt m_length = iPresencePublisher->Data()->iUserGiven.iDescription.iValue.MaxLength();

	TTime usergiventime;
	auto_ptr<HBufC> usergiven(HBufC::NewL( m_length) );
	TPtr16 p = usergiven->Des();
	if ( !Settings().GetSettingL(SETTING_OWN_DESCRIPTION, p) )  return;
	if ( !Settings().GetSettingL(SETTING_OWN_DESCRIPTION_TIME, usergiventime) ) return;

	auto_ptr<CBBSensorEvent> e(new CBBSensorEvent(KUserGiven, KUserGivenContextTuple, 0, GetTime()));
	TBBUserGiven * ug = new (ELeave) TBBUserGiven(KUserGiven);
	ug->iDescription.iValue = *usergiven;
	ug->iSince.iValue = usergiventime;
	e->iData() = ug;
	
	TTime expires; expires=Time::MaxTTime();
	CC_TRAPD(err, iBBSubSession->PutL(e->TupleName(), KNullDesC, e.get(), expires));
}

void CContext_logAppUi::SetUserGiven()
{
	CALLSTACKITEM_N(_CL("CContext_logAppUi"), _CL("SetUserGiven"));

	const TInt m_length = iPresencePublisher->Data()->iUserGiven.iDescription.iValue.MaxLength();

	TTime usergiventime;
	auto_ptr<HBufC> usergiven(HBufC::NewL( m_length) );
	TPtr16 p = usergiven->Des();
	Settings().GetSettingL(SETTING_OWN_DESCRIPTION, p);
	
	auto_ptr<HBufC> pr(CEikonEnv::Static()->AllocReadResourceL(R_CL_SET_USER_GIVEN));
	CAknTextQueryDialog* dlg = new(ELeave) CEmptyAllowingTextQuery(p, CAknQueryDialog::ENoTone);
	CleanupStack::PushL(dlg);
	dlg->SetPredictiveTextInputPermitted(ETrue);
	dlg->SetMaxLength( m_length );
	dlg->SetPromptL(*pr);
	
	CleanupStack::Pop();
	if (dlg->ExecuteLD(R_CONTEXT_LOG_SET_USER_GIVEN) ) {
		Settings().WriteSettingL(SETTING_OWN_DESCRIPTION_TIME, GetTime());
		Settings().WriteSettingL(SETTING_OWN_DESCRIPTION, p);
	}
}


class CLogEventTest : public CActive {
public:
	static CLogEventTest* NewL(RFs& Fs) {
		auto_ptr<CLogEventTest> ret(new (ELeave) CLogEventTest);
		ret->ConstructL(Fs);
		return ret.release();
	}
	CLogEventTest() : CActive(EPriorityNormal) { }
	void ConstructL(RFs& Fs) {
		lc=CLogClient::NewL(Fs);
		le=CLogEvent::NewL();
		le->SetEventType(KLogCallEventTypeUid);
		le->SetContact(1);
		le->SetDirection(_L("Missed call"));
		le->SetDuration(1);
		le->SetDurationType(KLogDurationValid);
		le->SetNumber(_L("0505536758"));
		TTime now; now.HomeTime();

		le->SetTime(now);
		le->SetRemoteParty(_L("mika"));
		CActiveScheduler::Add(this);
		lc->AddEvent(*le, iStatus);
		SetActive();
	}
	void RunL() {
		delete this;
	}
	void DoCancel() { }
	~CLogEventTest() {
		Cancel();
		delete lc;
		delete le;
	}
private:
	CLogClient*	lc;
	CLogEvent*	le;
};

void CContext_logAppUi::PresenceDetails()
{
	iPresenceDetailView->SetData(*iCaptionMyContext, iPresencePublisher->Data(), GetTime());

	ActivateLocalViewL(KPresenceDetailView);

	return;
}

void CContext_logAppUi::PresenceDescription()
{
	iPresenceDescriptionView->SetData(*iCaptionMyContext, iPresencePublisher->Data());

	ActivateLocalViewL(KPresenceDescriptionView);

	return;
}


void CContext_logAppUi::DialogTest()
{
	iSmsSnapshot->Test();
	return;

	iDumper=CBBDumper::NewL();
	return;

	{
		iSensorRunner->Stop();
		if (iLoggers) {
			iLoggers->ResetAndDestroy();
		}
		delete iLoggers; iLoggers=0;

		if (iMediaTest) delete iMediaTest;
		iMediaTest=0;
		iMediaTest=new (ELeave) CMediaTest(AppContext(), iMediaPrompt);
		iMediaTest->ConstructL();
	}
	return;

	delete iSocketTestReader; iSocketTestReader=0;
	iSocketTestReader=CSocketTestReader::NewL(AppContext(), 2000, _L("c:\\bblocal2.xml"));

	return;

	iDumper=CBBDumper::NewL();

	return;
	

	TInt ap; Settings().GetSettingL(SETTING_IP_AP, ap);
	iNTPConnection=CNTPConnection::NewL(*this);
	iNTPConnection->Sync(ap);
	return;

	/*
	_LIT(KS, "Liittymn 0407679267 sijainti: HELSINKI, Esplanadi, N60:10'06\", E24:56'57\" (ast:min'sek\" WGS84:ss)");
	TMsvId dummy(1);
	i_handle_received_sms* s=loc;
	s->handle_reception(dummy, dummy, _L("15400"), KS);
	return;
	_LIT(KS, "Liittymn 0407679267 sijainti: HELSINKI, Esplanadi, N60:10'06\", E24:56'57\" (ast:min'sek\" WGS84:ss)");
	TMsvId dummy(1);
	i_handle_received_sms* s=lsms;
	s->handle_sending(dummy, _L("0505536758"), KS);
	return;
	*/

	/*
	TMsvId dummy(1);
	iSmsSnapshot->handle_reception(dummy, dummy, _L("0505536758"), _L("WINS"));
	return;
	*/

	/*
	iSmsSnapshot->Test();
	return;
	*/

	CLogEventTest* t=CLogEventTest::NewL(Fs());
	
	return;

	/*
	TBuf<100> err;
	TInt ret=iMMS->SendMessage(_L("0505536758"), _L("body of msg"), _L("c:\\http.txt"), err, false);
	if (ret!=KErrNone) Error(ret, err);
	else status_change(_L("message sent"));
	return; */

	/**/

	//TBuf<2> buf; buf.Append(_L("xxxx"));

	if (iTestFileOpen) {
		iTestFile.Close();
		iTestFileOpen=false;
	} else {
		
		TFileName filen=_L("C:\\nokia\\images\\img");
		TInt i=2; filen.AppendNum(i); filen.Append(_L(".jpg"));
		while (BaflUtils::FileExists(Fs(), filen)) {
			filen=_L("C:\\nokia\\images\\img");
			++i; filen.AppendNum(i); filen.Append(_L(".jpg"));
		}
		{
			TFileName from=_L("C:\\nokia\\images\\img.jpg");
			BaflUtils::CopyFile(Fs(), from, filen);
		}
		{
			TFileName from2=_L("C:\\nokia\\images\\old\\img.jpg");
			BaflUtils::CopyFile(Fs(), from2, filen);
		}
		/*
		if (iTestFile.Open(Fs(), filen, EFileWrite)==KErrNone) {
			iTestFileOpen=true;
		}
		*/
	}
	return;

	/*
	if (IsDisplayingMenuOrDialog()) return;


	CCoeEnv *env = CEikonEnv::Static();
	RPbkViewResourceFile pbkRes( *env);
	pbkRes.OpenL();
	CleanupClosePushL(pbkRes);

	CPbkContactEngine *ipPabEng=CPbkContactEngine::NewL();
	CleanupStack::PushL(ipPabEng);

	CPbkContactItem* aContactItem = ipPabEng->ReadContactL( 1 );
	CleanupStack::PushL(aContactItem);

	// launch the contacts dialog
	CPbkContactEditorDlg *ipPabDlg =
	CPbkContactEditorDlg::NewL(*ipPabEng, *aContactItem, EFalse);

	ipPabDlg->SetMopParent( this );

	TInt res = KErrNone;
	CC_TRAPD( err, res = ipPabDlg->ExecuteLD());

	
	//CPbkSelectFieldDlg *d=new (ELeave) CPbkSelectFieldDlg;
	//TPbkContactItemField* res=d->ExecuteLD(aContactItem->CardFields(), R_AVKON_SOFTKEYS_OPTIONS_BACK);
	

	CleanupStack::PopAndDestroy(3); //engine, pbkRes, aContactItem
	*/
}

void CContext_logAppUi::NotifyNewPresence(const CBBPresence* d)
{
	CALLSTACKITEM_N(_CL("CContext_logAppUi"), _CL("NotifyNewPresence"));

	PresenceToListBoxL(d, iUserContext,
		iCaptionMyContext, 0, KNullDesC, GetTime(),
		prev, not_avail);

	iUserContextLog->AddL(*iUserContext);
}

void CContext_logAppUi::SettingChanged(TInt Setting)
{
	if (Setting==SETTING_LOGGING_ENABLE) {
		TApaTaskList tl(Ws());
		TApaTask booktask=tl.FindApp(KUidcontextbook);
		if (booktask.Exists()) {
			booktask.SendSystemEvent(EApaSystemEventShutdown);
		}
		TBool logging;
		Settings().GetSettingL(SETTING_LOGGING_ENABLE, logging);
		if (iLoggingRunning) {	
			if (!logging) {
				iLoggingRunning->SetCurrentState( EMbmContext_logL_not, EMbmContext_logL_not );
			} else {
				iLoggingRunning->SetCurrentState( EMbmContext_logL, EMbmContext_logL );
			}
		}

	} else if (Setting==SETTING_OWN_DESCRIPTION) {
		RestoreUserGiven();
	}
}
// End of File  
